local addon, private = ...
Mamigo = private

local function iif(condition, trueExpression, falseExpression)
	if condition then
		return trueExpression
	else
		return falseExpression
	end
end

local function ifnull(expression1, expression2)
	if expression1 == nil then
		return expression2
	else
		return expression1
	end
end

local function startsWith(s, start)
	return string.sub(s, 1, string.len(start)) == start
end

function merge(t1, t2)
	if t2 ~= nil then
		for k, v in pairs(t2) do
			if (type(v) == "table") and (type(t1[k] or false) == "table") then
				merge(t1[k], v)
			else
				t1[k] = v
			end
		end
	end
	return t1
end

local function dragDown(dragState, frame, event, ...)
	local mouse = Inspect.Mouse()
	dragState.dx = dragState.window:GetLeft() - mouse.x
	dragState.dy = dragState.window:GetTop() - mouse.y
	dragState.dragging = true
end

local function dragUp(dragState, frame, event, ...)
	dragState.dragging = false
end

local function dragMove(dragState, frame, event, x, y)
	if dragState.dragging then
		dragState.variable.x = x + dragState.dx
		dragState.variable.y = y + dragState.dy
		dragState.window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", dragState.variable.x, dragState.variable.y)
	end
end

local function dragAttach(window, variable)
	local dragState = { window = window, variable = variable, dragging = false }
	window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", dragState.variable.x, dragState.variable.y)
	window:EventAttach(Event.UI.Input.Mouse.Middle.Down, function(...) dragDown(dragState, ...) end, "dragDown")
	window:EventAttach(Event.UI.Input.Mouse.Middle.Up, function(...) dragUp(dragState, ...) end, "dragUp")
	window:EventAttach(Event.UI.Input.Mouse.Middle.Upoutside, function(...) dragUp(dragState, ...) end, "dragUpoutside")
	window:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(...) dragMove(dragState, ...) end, "dragMove")
end

local function createTexture(parent, x, y, w, h, texture, layout)
	local frame = UI.CreateFrame("Texture", "", parent)
	frame:SetPoint(ifnull(layout, "TOPLEFT"), parent, ifnull(layout, "TOPLEFT"), x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	if texture then	frame:SetTexture(addon.identifier, texture) end
	return frame
end

local function createText(parent, x, y, text)
	local frame = UI.CreateFrame("Text", "", parent)
	frame:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y)
	frame:SetFontSize(14)
	frame:SetText(text)
	return frame
end

local function createCheckbox(parent, x, y, text, value)
	local frame = {}

	frame.checkbox = UI.CreateFrame("RiftCheckbox", "", parent)
	frame.checkbox:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y + 2)
	frame.checkbox:SetEnabled(true)
	frame.checkbox:SetChecked(ifnull(value, false))

	frame.text = createText(parent, x + 16, y, text)
	frame.text:EventAttach(Event.UI.Input.Mouse.Left.Click, function (...)
		frame.checkbox:SetChecked(not frame.checkbox:GetChecked())
	end, "checkboxTextClick")

	return frame
end

local function createRadioButton(parent, x, y, text, value)
	local frame = {}

	frame.checkbox = UI.CreateFrame("RiftCheckbox", "", parent)
	frame.checkbox:SetPoint("TOPLEFT", parent, "TOPLEFT", x, y + 2)
	frame.checkbox:SetEnabled(true)
	frame.checkbox:SetChecked(ifnull(value, false))

	frame.text = createText(parent, x + 16, y, text)
	frame.text:EventAttach(Event.UI.Input.Mouse.Left.Click, function (...)
		if not frame.checkbox:GetChecked() then
			frame.checkbox:SetChecked(true)
		end
	end, "radioButtonTextClick")

	return frame
end

local function createButton(parent, align, x, y, w, h, text)
	local frame = UI.CreateFrame("RiftButton", "", parent)
	frame:SetPoint("TOPLEFT", align, "TOPLEFT", x, y)
	frame:SetWidth(w)
	frame:SetHeight(h)
	frame:SetText(text)
	return frame
end

local itemRarity = {
	common = 0,
	uncommon = 1,
	rare = 2,
	epic = 3,
	relic = 4,
	transcendent = 5,
	all = 6
}

local function dimensionMatchRarity(item, rarity)
	if item.rarity == nil then
		return true
	end
	if itemRarity[item.rarity] == nil then
		return false
	end
	return itemRarity[item.rarity] <= rarity
end

local dimensionState = {
	item = nil,
	setting = nil,
	setting_container = nil,
	setting_key = nil,
	queue = { }
}

local function dimensionMatchCategory(item, category)
	if item.category == nil then
		return false
	end
	if startsWith(item.category, "dimension key") then
		return dimensionState.setting_key
	end
	if startsWith(item.category, "dimension") and item.lootable then
		return dimensionState.setting_container
	end
	return startsWith(item.category, category)
end

local function dimensionMatch()
	if not Inspect.Queue.Status("global") then
		return nil
	end

	-- Checking items is slow, so don't do it if nothing has changed
	if dimensionState.setting == MamigoGlobal.settings.dimension
		and dimensionState.setting_container == MamigoGlobal.settings.dimension_container
		and dimensionState.setting_key == MamigoGlobal.settings.dimension_key then
		return dimensionState.item
	end
	dimensionState.item = nil
	dimensionState.setting = MamigoGlobal.settings.dimension
	dimensionState.setting_container = MamigoGlobal.settings.dimension_container
	dimensionState.setting_key = MamigoGlobal.settings.dimension_key

	local items = Inspect.Item.Detail(Utility.Item.Slot.Inventory())
	if items == nil then
		return nil
	end

	local category = "dimension"
	local rarity = nil
	if dimensionState.setting == "none" then
		return nil
	elseif dimensionState.setting == "block" then
		category = "dimension block"
		rarity = itemRarity.common
	else
		rarity = itemRarity[dimensionState.setting]
	end
	if rarity == nil then
		return nil
	end

	for k, v in pairs(items) do
		if dimensionMatchRarity(v, rarity) and dimensionMatchCategory(v, category) then
			dimensionState.item = v
			return v
		end
	end

	return nil
end

local function dimensionDestroy()
	local item = dimensionMatch()
	if item == nil then
		return false
	end

	print("Destroying " .. item.name)
	Command.Item.Destroy(item.id)
	dimensionState.setting = nil
	return true
end

local statNames = {
	"Air",
	"Artifact",
	"Assassination",
	"Death",
	"Dimension",
	"Diplomacy",
	"Earth",
	"Exploration",
	"Fire",
	"Harvest",
	"Hunting",
	"Life",
	"Water"
}

local function minionMatch(adventure, minion)
	local sort = MamigoGlobal.settings.sort
	local max = ifnull(MamigoGlobal.settings.max[adventure.reward], MamigoGlobal.settings.max.other)
	local min = MamigoGlobal.settings.min
	if adventure.reward == "experience" then
		sort = MamigoGlobal.settings.sort_exp
		max = false
	end

	if not min and minion.level == 1 then
		return -1
	end

	if not max and minion.experienceNeeded == nil then
		return -1
	end

	local stamina = adventure.costStamina
	if (minion.experienceNeeded == nil or not MamigoGlobal.settings.stamina_max_only) and  stamina < MamigoGlobal.settings.stamina then
		stamina = stamina + MamigoGlobal.settings.stamina
	end
	if minion.stamina < stamina then
		return -1
	end

	local stat = 0
	for i, name in ipairs(statNames) do
		if adventure["stat" .. name] and minion["stat" .. name] ~= nil then
			if sort == "stamina" then
				stat = minion.stamina
			elseif sort == "stat" then
				stat = stat + minion["stat" .. name]
			elseif sort == "level" then
				stat = minion.level
			elseif sort == "levelasc" then
				stat = 1000 - minion.level
			end
		end
	end
	-- Give preference to minions that aren't at the level cap.
	if minion.experienceNeeded ~= nil then
		stat = stat + 0.5
	end
	return stat
end

local function minionLocked(name)
	for k, v in pairs(MamigoGlobal.settings.lock) do
		if k == name then
			return true
		end
	end
	return false
end

local mamigoButton = nil
local updateEnable = 0
local function mamigoEnable(enable)
	mamigoButton:SetEnabled(enable)
	updateEnable = Inspect.Time.Real()
end

local function minionReady()
	local aids = Inspect.Minion.Adventure.List()
	if aids == nil then
		-- This can happen when logging in
		mamigoEnable(false)
		return
	end
	local adventures = Inspect.Minion.Adventure.Detail(aids)
	local slot = Inspect.Minion.Slot()
	for aid, adventure in pairs(adventures) do
		if adventure.mode == "finished" and not minionLocked(Inspect.Minion.Minion.Detail(adventure.minion).name) then
			mamigoEnable(true)
			return
		end
		if adventure.mode == "finished" or adventure.mode == "working" then
			slot = slot - 1
		end
	end
	if slot > 0 then
		mamigoEnable(true)
		return
	end

	mamigoEnable(dimensionMatch() ~= nil)
end

local function minionReadyTimer()
	if (Inspect.Time.Real() >= updateEnable + 1) then
		minionReady()
	end
end

local function itemReady()
	-- force a dimension item check
	dimensionState.setting = nil
	minionReady()
end

local function minionSend(aid, adventure, busy)
	local mids = Inspect.Minion.Minion.List()
	local minions = Inspect.Minion.Minion.Detail(mids)
	local best = -1
	local bestid = false
	local bestminion = nil
	for mid, minion in pairs(minions) do
		if busy[mid] == nil and not minionLocked(minion.name) then
			local stat = minionMatch(adventure, minion)
			if stat >= 0 and (best < stat or best == stat and bestminion.stamina < minion.stamina) then
				best = stat
				bestid = mid
				bestminion = minion
			end
		end
	end
	if best >= 0 then
		print("Sending " .. bestminion.name .. " on " .. adventure.name)
		if adventure.costAventurine > 0 then
			Command.Minion.Send(bestid, aid, "aventurine")
		else
			Command.Minion.Send(bestid, aid, "none")
		end
	else
		print("No available minions")
	end
end

local adventureMatch = {
	experience = function (a) return a.reward == "experience" and a.costAventurine == 0 end,
	short = function (a) return (a.duration == 5*60 or a.duration == 15*60) and a.reward ~= "experience" and a.costAventurine == 0 end,
	long = function (a) return a.duration == 8*60*60 and a.reward ~= "experience" and a.costAventurine == 0 end,
	aventurine = function (a) return a.duration == 10*60*60 and a.reward ~= "experience" and a.costAventurine > 0 end
}

local adventureMatchAll = {
	experience = function (a) return a.reward == "experience" and a.costAventurine == 0 end,
	short = function (a) return a.duration < 2*60*60 and a.reward ~= "experience" and a.costAventurine == 0 end,
	long = function (a) return a.duration >= 2*60*60 and a.reward ~= "experience" and a.costAventurine == 0 end,
	aventurine = function (a) return a.duration >= 2*60*60 and a.reward ~= "experience" and a.costAventurine > 0 end
}

local function minionGo()
	if not mamigoButton:GetEnabled() then
		return
	end
	mamigoEnable(false)

	local aids = Inspect.Minion.Adventure.List()
	local adventures = Inspect.Minion.Adventure.Detail(aids)
	local slot = Inspect.Minion.Slot()
	local busy = {}
	for aid, adventure in pairs(adventures) do
		if adventure.mode == "finished" and not minionLocked(Inspect.Minion.Minion.Detail(adventure.minion).name) then
			print("Claiming " .. adventure.name)
			Command.Minion.Claim(aid)
			return
		end
		if adventure.mode == "finished" or adventure.mode == "working" then
			slot = slot - 1
			busy[adventure.minion] = adventure
		end
	end

	if slot <= 0 then
		if dimensionDestroy() then
			return
		end

		print("No available slots")
		return
	end

	local match
	if MamigoGlobal.settings.adventure_all then
		match = adventureMatchAll[MamigoGlobal.settings.adventure]
	else
		match = adventureMatch[MamigoGlobal.settings.adventure]
	end
	for aid, adventure in pairs(adventures) do
		if adventure.mode == "available" and match(adventure) then
			minionSend(aid, adventure, busy)
			return
		end
	end
	print("No available adventures")
end

local mamigoXp = 0
local function minionUp()
	local xp = 0
	local mids = Inspect.Minion.Minion.List()
	local minions = Inspect.Minion.Minion.Detail(mids)
	for mid, minion in pairs(minions) do
		xp = xp + (minion.level - 1) * (minion.level * 150 - 100) + ifnull(minion.experienceAccumulated, 0)
	end
	if mamigoXp < xp then
		mamigoXp = xp
		Command.Console.Display("general", true, "<font color=\"#FFFF00\">The Mamigo number is : " .. tostring(mamigoXp) .. "</font>", true)
	end
end

local function showSubmenu(menu, menuitem, submenu)
	menuitem:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (...)
		if menu.submenu ~= nil then
			menu.submenu:SetVisible(false)
		end
		menu.submenu = submenu
		submenu:SetVisible(true)
	end, "showSubmenu")
end

local function hideSubmenu(menu, menuitem)
	menuitem:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (...)
		if menu.submenu ~= nil then
			menu.submenu:SetVisible(false)
			menu.submenu = nil
		end
	end, "hideSubmenu")
end

local function createMenuFrame(parent, y)
	local frame = UI.CreateFrame("Frame", "", parent)
	frame:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, y)
	frame:SetWidth(170)
	frame:SetHeight(20)
	return frame
end

local function createMenuCheckbox(parent, y, text, settings, key)
	local frame = createMenuFrame(parent, y)
	local control = createCheckbox(frame, 22, 0, text, settings[key])
	control.checkbox:EventAttach(Event.UI.Checkbox.Change, function (...)
		settings[key] = control.checkbox:GetChecked()
	end, "menuCheckboxChange")
	hideSubmenu(parent, frame)
	hideSubmenu(parent, control.text)
	return y + 20
end

local function createMenuRadioButton(parent, y, text, settings, key, value, group)
	local frame = createMenuFrame(parent, y)
	local control = createRadioButton(frame, 22, 0, text, settings[key] == value)
	control.checkbox:EventAttach(Event.UI.Checkbox.Change, function (...)
		if control.checkbox:GetChecked() then
			settings[key] = value
			for k, v in pairs(group) do
				if k ~= value then
					v.checkbox:SetChecked(false)
				end
			end
		end
	end, "menuRadioButtonChange")
	hideSubmenu(parent, frame)
	group[value] = control
	return y + 20
end

local function createMenuSeparator(parent, y)
	local control = createTexture(parent, 5, y + 10, 190, 5, "img/tooltip-separator.png", "TOPLEFT")
	hideSubmenu(parent, control)
	return y + 20
end

local function createMenuHeader(parent, y, text)
	local frame = createMenuFrame(parent, y)
	createText(frame, 20, 0, text)
	hideSubmenu(parent, frame)
	return y + 20
end

local function createMenu(parent, cb)
	local window = UI.CreateFrame("Frame", "", parent)
	window:SetWidth(200)
	createTexture(window, 0, 0, 200, 12, "img/tooltip200-top.png", "TOPLEFT"):SetLayer(-1)
	createTexture(window, 0, 0, 200, 26, "img/tooltip200-bottom.png", "BOTTOMLEFT"):SetLayer(-1)
	local body = createTexture(window, 0, 12, 200, 0, "img/tooltip200-middle.png", "TOPLEFT")
	local y = 10
	y = cb(body, y)
	body:SetHeight(y + 10)
	window:SetHeight(y + 48)

	-- stop clicks on elements behind the menu
	window:EventAttach(Event.UI.Input.Mouse.Left.Click, function (...)
	end, "menuLeftClick")

	return window
end

local function createSubmenu(parent, y, text, cb)
	local frame = createMenuFrame(parent, y)
	createText(frame, 38, 0, text)

	local child = createMenu(frame, cb)
	child:SetVisible(false)
	child:SetPoint("TOPLEFT", frame, "TOPRIGHT", -10, -20)
	frame:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (...)
		if parent.submenu ~= nil then
			parent.submenu:SetVisible(false)
		end
		parent.submenu = child
		child:SetVisible(true)
	end, "submenuCursorIn")
	return y + 20
end

local function menuInit(parent, settings)
	local window = createMenu(parent, function (body, y)
		y = createSubmenu(body, y, "Adventure type:", function (body, y)
			local adventure = {}
			y = createMenuRadioButton(body, y, "Experience", settings, "adventure", "experience", adventure)
			y = createMenuRadioButton(body, y, "5/15 minute", settings, "adventure", "short", adventure)
			y = createMenuRadioButton(body, y, "8 hour", settings, "adventure", "long", adventure)
			y = createMenuRadioButton(body, y, "10 hour", settings, "adventure", "aventurine", adventure)
			y = createMenuSeparator(body, y)
			y = createMenuCheckbox(body, y, "Include special", settings, "adventure_all")
			return y
		end)

		y = createSubmenu(body, y, "Sort by:", function (body, y)
			local sort = {}
			y = createMenuRadioButton(body, y, "Stamina", settings, "sort", "stamina", sort)
			y = createMenuRadioButton(body, y, "Stat", settings, "sort", "stat", sort)
			y = createMenuRadioButton(body, y, "Level (desc)", settings, "sort", "level", sort)
			y = createMenuRadioButton(body, y, "Level (asc)", settings, "sort", "levelasc", sort)
			return y
		end)

		y = createSubmenu(body, y, "Experience sort by:", function (body, y)
			local sort = {}
			y = createMenuRadioButton(body, y, "Stamina", settings, "sort_exp", "stamina", sort)
			y = createMenuRadioButton(body, y, "Stat", settings, "sort_exp", "stat", sort)
			y = createMenuRadioButton(body, y, "Level (desc)", settings, "sort_exp", "level", sort)
			y = createMenuRadioButton(body, y, "Level (asc)", settings, "sort_exp", "levelasc", sort)
			return y
		end)

		y = createSubmenu(body, y, "Reserve stamina for:", function (body, y)
			local stamina = {}
			y = createMenuRadioButton(body, y, "None", settings, "stamina", 0, stamina)
			y = createMenuRadioButton(body, y, "8 hour", settings, "stamina", 7, stamina)
			y = createMenuRadioButton(body, y, "10 hour", settings, "stamina", 10, stamina)
			y = createMenuSeparator(body, y)
			y = createMenuCheckbox(body, y, "Level 25 only", settings, "stamina_max_only")
			return y
		end)

		y = createSubmenu(body, y, "Send level 25 for:", function (body, y)
			y = createMenuCheckbox(body, y, "Artifact", settings.max, "artifact")
			y = createMenuCheckbox(body, y, "Dimension", settings.max, "dimension")
			y = createMenuCheckbox(body, y, "Diplomacy", settings.max, "diplomacy")
			y = createMenuCheckbox(body, y, "Harvest", settings.max, "harvest")
			y = createMenuCheckbox(body, y, "Hunt", settings.max, "hunt")
			y = createMenuCheckbox(body, y, "Material", settings.max, "material")
			y = createMenuCheckbox(body, y, "Other", settings.max, "other")
			return y
		end)

		y = createMenuCheckbox(body, y, "Send level 1", settings, "min")

		y = createSubmenu(body, y, "Destroy dimension:", function (body, y)
			local dimension = {}
			y = createMenuRadioButton(body, y, "None", settings, "dimension", "none", dimension)
			y = createMenuRadioButton(body, y, "Blocks", settings, "dimension", "block", dimension)
			y = createMenuRadioButton(body, y, "Common", settings, "dimension", "common", dimension)
			y = createMenuRadioButton(body, y, "Uncommon", settings, "dimension", "uncommon", dimension)
			y = createMenuRadioButton(body, y, "Rare", settings, "dimension", "rare", dimension)
			y = createMenuRadioButton(body, y, "All", settings, "dimension", "all", dimension)
			y = createMenuSeparator(body, y)
			y = createMenuCheckbox(body, y, "Container", settings, "dimension_container")
			y = createMenuCheckbox(body, y, "Key", settings, "dimension_key")
			return y
		end)

		y = createMenuCheckbox(body, y, "Flash", settings, "flash")

		return y
	end)
	return window
end

local function menuToggle(align, window)
	if align:GetTop() > UIParent:GetHeight() / 2 then
		window:ClearPoint("TOPCENTER")
		window:SetPoint("BOTTOMCENTER", align, "TOPCENTER", 0, 20)
	else
		window:ClearPoint("BOTTOMCENTER")
		window:SetPoint("TOPCENTER", align, "BOTTOMCENTER", 0, 0)
	end
	window:SetVisible(not window:GetVisible())
end

local function createMamigoButton(context)
	local frame = UI.CreateFrame("Frame", "", context)
	frame:SetWidth(32)
	frame:SetHeight(32)
	frame:SetLayer(1)

	local texture1 = createTexture(frame, 0, 0, 64, 64, nil, "CENTER")
	local texture2 = createTexture(frame, 0, 0, 24, 24, nil, "CENTER")
	texture2:SetLayer(1)
	local texture3 = createTexture(frame, -4, -4, 64, 64, nil, "TOPLEFT")
	texture3:SetTexture("Rift", "death_icon_(glow).png.dds")
	texture3:SetLayer(3)

	frame:EventAttach(Event.UI.Input.Mouse.Left.Down, function (...)
		if frame.state then
			texture1:SetTexture("Rift", "btn_zoomout_(click).png.dds")
			texture2:SetTexture("Rift", "icon_menu_minion_(select).png.dds")
			texture2:SetPoint("CENTER", frame, "CENTER", 0, 2)
		end
	end, "buttonLeftDown")

	frame:EventAttach(Event.UI.Input.Mouse.Left.Up, function (...)
		if frame.state then
			texture1:SetTexture("Rift", "btn_zoomout_(over).png.dds")
			texture2:SetTexture("Rift", "icon_menu_minion_(select).png.dds")
			texture2:SetPoint("CENTER", frame, "CENTER", 0, 0)
		end
	end, "buttonLeftUp")

	frame:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (...)
		if frame.state then
			texture1:SetTexture("Rift", "btn_zoomout_(over).png.dds")
			texture2:SetTexture("Rift", "icon_menu_minion_(select).png.dds")
			texture2:SetPoint("CENTER", frame, "CENTER", 0, 0)
		end
	end, "buttonCursorIn")

	frame:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (...)
		if frame.state then
			texture1:SetTexture("Rift", "btn_zoomout_(normal).png.dds")
			texture2:SetTexture("Rift", "icon_menu_minion.png.dds")
			texture2:SetPoint("CENTER", frame, "CENTER", 0, 0)
		end
	end, "buttonCursorOut")

	function frame:SetEnabled(enabled)
		if frame.state ~= enabled then
			frame.state = enabled
			if frame.state then
				texture1:SetTexture("Rift", "btn_zoomout_(normal).png.dds")
				texture2:SetTexture("Rift", "icon_menu_minion.png.dds")
				texture2:SetPoint("CENTER", frame, "CENTER", 0, 0)
			else
				texture1:SetTexture("Rift", "btn_zoomout_(disable).png.dds")
				texture2:SetTexture("Rift", "icon_menu_minion_(disabled).png.dds")
				texture2:SetPoint("CENTER", frame, "CENTER", 0, 0)
			end
			texture3:SetVisible(frame.state)
		end
	end

	function frame:GetEnabled(enabled)
		return frame.state
	end

	Command.Event.Attach(Event.System.Update.Begin, function ()
		if MamigoGlobal.settings.flash then
			texture3:SetAlpha(math.abs(Inspect.Time.Real() * 20 % 20 - 10) / 10)
		else
			texture3:SetAlpha(0)
		end
	end, "buttonAnimate")

	frame:SetEnabled(false)
	return frame
end

local function settingsInit()
	if MamigoGlobal == nil then MamigoGlobal = {} end
	MamigoGlobal.settings = merge({
		adventure = "experience",
		sort = "stat",
		sort_exp = "stamina",
		stamina = 0,
		max = {
			artifact = true,
			dimension = true,
			diplomacy = true,
			harvest = true,
			hunt = true,
			material = true,
			other = true
		},
		min = false,
		dimension = "none",
		lock = {
		},
		flash = false
	}, MamigoGlobal.settings)

	MamigoSettings = merge({
		window = {
			x = math.floor(UIParent:GetWidth() / 4),
			y = math.floor(UIParent:GetHeight() / 4)
		}
	}, MamigoSettings)
end

local function addonSlash(event, params)
	if params == "lock" then
		print("Locked minions:")
		for k, v in pairs(MamigoGlobal.settings.lock) do
			print(k)
		end
	elseif startsWith(params, "lock ") then
		local minion = params:sub(6):lower()
		for k, v in pairs(Inspect.Minion.Minion.Detail(Inspect.Minion.Minion.List())) do
			if v.name:lower() == minion then
				MamigoGlobal.settings.lock[v.name] = true
				print("Locked " .. v.name)
				return
			end
		end
		print("Unknown minion")
	elseif startsWith(params, "unlock ") then
		local minion = params:sub(8):lower()
		for k, v in pairs(MamigoGlobal.settings.lock) do
			if k:lower() == minion then
				MamigoGlobal.settings.lock[k] = nil
				print("Unlocked " .. k)
				return
			end
		end
	else
		print("Left mouse: send/claim")
		print("Middle mouse: move button")
		print("Right mouse: menu")
		print("Commands:")
		print("/" .. addon.identifier .. " lock - list locked minions")
		print("/" .. addon.identifier .. " lock {minion} - lock minion")
		print("/" .. addon.identifier .. " unlock {minion} - unlock minion")
	end
end

local function addonInit(handle, addonIdentifier)
	if addonIdentifier ~= addon.identifier then
		return
	end

	settingsInit()

	local context = UI.CreateContext(addon.identifier)

	mamigoButton = createMamigoButton(context)
	dragAttach(mamigoButton, MamigoSettings.window)
	mamigoButton:EventAttach(Event.UI.Input.Mouse.Left.Click, minionGo, "minionGo")
	mamigoButton:EventAttach(Event.UI.Input.Mouse.Wheel.Back, minionGo, "minionGoBack")
	mamigoButton:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, minionGo, "minionGoForward")

	local mamigoMenu = menuInit(context, MamigoGlobal.settings)
	mamigoMenu:SetVisible(false)
	mamigoMenu:SetPoint("BOTTOMCENTER", mamigoButton, "TOPCENTER", 0, 20)
	mamigoButton:EventAttach(Event.UI.Input.Mouse.Right.Click, function ()
		menuToggle(mamigoButton, mamigoMenu)
	end, "menuRightClick")

	Command.Event.Attach(Command.Slash.Register(addon.identifier), addonSlash, "addonSlash")
	Command.Event.Attach(Event.System.Update.Begin, minionReadyTimer, "minionReadyTimer")
	Command.Event.Attach(Event.Minion.Adventure.Change, minionReady, "minionReady")
	Command.Event.Attach(Event.Minion.Minion.Change, minionUp, "minionUp")
	Command.Event.Attach(Event.Item.Slot, itemReady, "itemReady")
	Command.Event.Attach(Event.Queue.Status, minionReady, "minionReady")
end

Command.Event.Attach(Event.Addon.Load.End, addonInit, "addonInit")
